@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,30 +23,64 @@
 Actions that perform cleaning
 """
 
+from __future__ import absolute_import, print_function
+
+from bleachbit import Command, FileUtilities, General, Special
+from bleachbit import _, expanduser, expandvars
 
 import glob
 import logging
 import os
 import re
 import types
-import Command
-import FileUtilities
-import General
-import Special
 
-from Common import _
 
 if 'posix' == os.name:
     re_flags = 0
-    import Unix
+    from bleachbit import Unix
 else:
     re_flags = re.IGNORECASE
 
+logger = logging.getLogger(__name__)
+
+
+def has_glob(s):
+    """Checks whether the string contains any glob characters"""
+    return re.search('[?*\[\]]', s) is not None
+
+
+def expand_multi_var(s, variables):
+    """Expand string s with potentially-multiple values.
+
+    The placeholder is written in the format $$foo$$.
+
+    The function always returns a list of one or more strings.
+    """
+    if not variables or s.find('$$') == -1:
+        # The input string is missing $$ or no variables are given.
+        return (s,)
+    matched = False
+    ret = []
+    for var_key, var_values in variables.iteritems():
+        sub = '$$%s$$' % var_key
+        if s.find(sub) == -1:
+            continue
+        matched = True
+        for var_value in var_values:
+            modified = s.replace(sub, var_value)
+            ret.append(modified)
+    if matched:
+        return ret
+    else:
+        # The string has $$, but it did not match anything
+        return (s,)
 
 #
 # Plugin framework
 # http://martyalchin.com/2008/jan/10/simple-plugin-framework/
 #
+
+
 class PluginMount(type):
 
     """A simple plugin framework"""
@@ -63,7 +97,7 @@ class ActionProvider:
     """Abstract base class for performing individual cleaning actions"""
     __metaclass__ = PluginMount
 
-    def __init__(self, action_node):
+    def __init__(self, action_node, path_vars=None):
         """Create ActionProvider from CleanerML <action>"""
         pass
 
@@ -84,7 +118,7 @@ class FileActionProvider(ActionProvider):
     """Base class for providers which work on individual files"""
     action_key = '_file'
 
-    def __init__(self, action_element):
+    def __init__(self, action_element, path_vars=None):
         """Initialize file search"""
         self.regex = action_element.getAttribute('regex')
         assert(isinstance(self.regex, (str, unicode, types.NoneType)))
@@ -96,13 +130,7 @@ class FileActionProvider(ActionProvider):
         assert(isinstance(self.nwholeregex, (str, unicode, types.NoneType)))
         self.search = action_element.getAttribute('search')
         self.object_type = action_element.getAttribute('type')
-        self.path = os.path.expanduser(FileUtilities.expandvars(
-            action_element.getAttribute('path')))
-        if 'nt' == os.name and self.path:
-            # convert forward slash to backslash for compatibility with getsize()
-            # and for display.  Do not convert an empty path, or it will become
-            # the current directory (.).
-            self.path = os.path.normpath(self.path)
+        self._set_paths(action_element.getAttribute('path'), path_vars)
         self.ds = {}
         if 'deep' == self.search:
             self.ds['regex'] = self.regex
@@ -110,12 +138,28 @@ class FileActionProvider(ActionProvider):
             self.ds['cache'] = General.boolstr_to_bool(
                 action_element.getAttribute('cache'))
             self.ds['command'] = action_element.getAttribute('command')
-            self.ds['path'] = self.path
+            self.ds['path'] = self.paths[0]
+            if not len(self.paths) == 1:
+                logger.warning(
+                    _("Deep scan does not support multi-value variable."))
         if not any([self.object_type, self.regex, self.nregex,
                     self.wholeregex, self.nwholeregex]):
             # If the filter is not needed, bypass it for speed.
             self.get_paths = self._get_paths
 
+    def _set_paths(self, raw_path, path_vars):
+        """Set the list of paths to work on"""
+        self.paths = []
+        # expand special $$foo$$ which may give multiple values
+        for path2 in expand_multi_var(raw_path, path_vars):
+            path3 = expanduser(expandvars(path2))
+            if os.name == 'nt' and path3:
+                # convert forward slash to backslash for compatibility with getsize()
+                # and for display.  Do not convert an empty path, or it will become
+                # the current directory (.).
+                path3 = os.path.normpath(path3)
+            self.paths.append(path3)
+
     def get_deep_scan(self):
         if 0 == len(self.ds):
             raise StopIteration
@@ -165,14 +209,20 @@ class FileActionProvider(ActionProvider):
 
         def get_walk_all(top):
             for expanded in glob.iglob(top):
+                any_match = False
                 for path in FileUtilities.children_in_directory(
                         expanded, True):
+                    any_match = True
                     yield path
+                # This is a lint checker because this scenario may
+                # indicate the cleaner developer made a mistake.
+                if not any_match and os.path.isfile(expanded):
+                    logger.debug(
+                        _('search="walk.all" used with regular file path="%s"'), expanded)
 
         def get_walk_files(top):
             for expanded in glob.iglob(top):
-                for path in FileUtilities.children_in_directory(
-                        expanded, False):
+                for path in FileUtilities.children_in_directory(expanded, False):
                     yield path
 
         if 'deep' == self.search:
@@ -200,8 +250,12 @@ class FileActionProvider(ActionProvider):
         if self.nwholeregex:
             self.nwholeregex_c = re.compile(self.nwholeregex, re_flags)
 
-        for path in func(self.path):
-            yield path
+        for input_path in self.paths:
+            if self.search == 'glob' and not has_glob(input_path):
+                # lint for people who develop cleaners
+                logger.debug(_('path="%s" is not a glob pattern'), input_path)
+            for path in func(input_path):
+                yield path
 
     def get_commands(self):
         raise NotImplementedError('not implemented')
@@ -217,7 +271,7 @@ class AptAutoclean(ActionProvider):
     """Action to run 'apt-get autoclean'"""
     action_key = 'apt.autoclean'
 
-    def __init__(self, action_element):
+    def __init__(self, action_element, path_vars=None):
         pass
 
     def get_commands(self):
@@ -233,7 +287,7 @@ class AptAutoremove(ActionProvider):
     """Action to run 'apt-get autoremove'"""
     action_key = 'apt.autoremove'
 
-    def __init__(self, action_element):
+    def __init__(self, action_element, path_vars=None):
         pass
 
     def get_commands(self):
@@ -244,6 +298,22 @@ class AptAutoremove(ActionProvider):
                                    'apt-get autoremove')
 
 
+class AptClean(ActionProvider):
+
+    """Action to run 'apt-get clean'"""
+    action_key = 'apt.clean'
+
+    def __init__(self, action_element, path_vars=None):
+        pass
+
+    def get_commands(self):
+        # Checking executable allows auto-hide to work for non-APT systems
+        if FileUtilities.exe_exists('apt-get'):
+            yield Command.Function(None,
+                                   Unix.apt_clean,
+                                   'apt-get clean')
+
+
 class ChromeAutofill(FileActionProvider):
 
     """Action to clean 'autofill' table in Google Chrome/Chromium"""
@@ -324,8 +394,8 @@ class Ini(FileActionProvider):
     """Action to clean .ini configuration files"""
     action_key = 'ini'
 
-    def __init__(self, action_element):
-        FileActionProvider.__init__(self, action_element)
+    def __init__(self, action_element, path_vars=None):
+        FileActionProvider.__init__(self, action_element, path_vars)
         self.section = action_element.getAttribute('section')
         self.parameter = action_element.getAttribute('parameter')
         if self.parameter == "":
@@ -336,13 +406,25 @@ class Ini(FileActionProvider):
             yield Command.Ini(path, self.section, self.parameter)
 
 
+class Journald(ActionProvider):
+    """Action to run 'journalctl --vacuum-time=1'"""
+    action_key = 'journald.clean'
+
+    def __init__(self, action_element, path_vars=None):
+        pass
+
+    def get_commands(self):
+        if FileUtilities.exe_exists('journalctl'):
+            yield Command.Function(None, Unix.journald_clean, 'journalctl --vacuum-time=1')
+
+
 class Json(FileActionProvider):
 
     """Action to clean JSON configuration files"""
     action_key = 'json'
 
-    def __init__(self, action_element):
-        FileActionProvider.__init__(self, action_element)
+    def __init__(self, action_element, path_vars=None):
+        FileActionProvider.__init__(self, action_element, path_vars)
         self.address = action_element.getAttribute('address')
 
     def get_commands(self):
@@ -353,11 +435,13 @@ class Json(FileActionProvider):
 class MozillaUrlHistory(FileActionProvider):
 
     """Action to clean Mozilla (Firefox) URL history in places.sqlite"""
-    action_key = 'mozilla_url_history'
+    action_key = 'mozilla.url.history'
 
     def get_commands(self):
         for path in self.get_paths():
-            yield Special.delete_mozilla_url_history(path)
+            yield Command.Function(path,
+                                   Special.delete_mozilla_url_history,
+                                   _('Clean file'))
 
 
 class OfficeRegistryModifications(FileActionProvider):
@@ -378,8 +462,8 @@ class Process(ActionProvider):
     """Action to run a process"""
     action_key = 'process'
 
-    def __init__(self, action_element):
-        self.cmd = FileUtilities.expandvars(action_element.getAttribute('cmd'))
+    def __init__(self, action_element, path_vars=None):
+        self.cmd = expandvars(action_element.getAttribute('cmd'))
         # by default, wait
         self.wait = True
         wait = action_element.getAttribute('wait')
@@ -398,15 +482,13 @@ class Process(ActionProvider):
                     rc = 0  # unknown because we don't wait
                     from subprocess import Popen
                     Popen(self.cmd)
-            except Exception, e:
+            except Exception as e:
                 raise RuntimeError(
                     'Exception in external command\nCommand: %s\nError: %s' % (self.cmd, str(e)))
             else:
                 if not 0 == rc:
-                    logger = logging.getLogger(__name__)
-                    logger.warning(
-                        'Command: %s\nReturn code: %d\nStdout: %s\nStderr: %s\n' %
-                        (self.cmd, rc, stdout, stderr))
+                    logger.warning('Command: %s\nReturn code: %d\nStdout: %s\nStderr: %s\n',
+                                   self.cmd, rc, stdout, stderr)
             return 0
         yield Command.Function(path=None, func=run_process, label=_("Run external command: %s") % self.cmd)
 
@@ -454,7 +536,7 @@ class WinShellChangeNotify(ActionProvider):
     action_key = 'win.shell.change.notify'
 
     def get_commands(self):
-        import Windows
+        from bleachbit import Windows
         yield Command.Function(
             None,
             Windows.shell_change_notify,
@@ -466,7 +548,7 @@ class Winreg(ActionProvider):
     """Action to clean the Windows Registry"""
     action_key = 'winreg'
 
-    def __init__(self, action_element):
+    def __init__(self, action_element, path_vars=None):
         self.keyname = action_element.getAttribute('path')
         self.name = action_element.getAttribute('name')
 
@@ -479,7 +561,7 @@ class YumCleanAll(ActionProvider):
     """Action to run 'yum clean all'"""
     action_key = 'yum.clean_all'
 
-    def __init__(self, action_element):
+    def __init__(self, action_element, path_vars=None):
         pass
 
     def get_commands(self):
