@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,21 @@
 Command design pattern implementation for cleaning
 """
 
+from __future__ import absolute_import, print_function
 
+from bleachbit import _
+from bleachbit import FileUtilities
+
+import logging
 import os
 import types
-import FileUtilities
 
 from sqlite3 import DatabaseError
-from Common import _
 
 if 'nt' == os.name:
-    import Windows
+    import bleachbit.Windows
 else:
-    from General import WindowsError
+    from bleachbit.General import WindowsError
 
 
 def whitelist(path):
@@ -79,17 +82,21 @@ class Delete:
         if really_delete:
             try:
                 FileUtilities.delete(self.path, self.shred)
-            except WindowsError, e:
+            except WindowsError as e:
                 # WindowsError: [Error 32] The process cannot access the file because it is being
                 # used by another process: u'C:\\Documents and
                 # Settings\\username\\Cookies\\index.dat'
                 if 32 != e.winerror and 5 != e.winerror:
                     raise
                 try:
-                    Windows.delete_locked_file(self.path)
+                    bleachbit.Windows.delete_locked_file(self.path)
                 except:
                     raise
                 else:
+                    if self.shred:
+                        import warnings
+                        warnings.warn(
+                            _('At least one file was locked by another process, so its contents could not be overwritten. It will be marked for deletion upon system reboot.'))
                     # TRANSLATORS: The file will be deleted when the
                     # system reboots
                     ret['label'] = _('Mark for deletion')
@@ -112,9 +119,15 @@ class Function:
         except AssertionError:
             raise AssertionError('Expected MethodType but got %s' % type(func))
 
+    def __str__(self):
+        if self.path:
+            return 'Function: %s: %s' % (self.label, self.path)
+        else:
+            return 'Function: %s' % (self.label)
+
     def execute(self, really_delete):
 
-        if None != self.path and FileUtilities.whitelisted(self.path):
+        if self.path is not None and FileUtilities.whitelisted(self.path):
             yield whitelist(self.path)
             return
 
@@ -126,7 +139,7 @@ class Function:
             'size': None}
 
         if really_delete:
-            if None == self.path:
+            if self.path is None:
                 # Function takes no path.  It returns the size.
                 func_ret = self.func()
                 if isinstance(func_ret, types.GeneratorType):
@@ -147,15 +160,15 @@ class Function:
                 oldsize = FileUtilities.getsize(self.path)
                 try:
                     self.func(self.path)
-                except DatabaseError, e:
+                except DatabaseError as e:
                     if -1 == e.message.find('file is encrypted or is not a database') and \
                        -1 == e.message.find('or missing database'):
                         raise
-                    print 'Warning:', e.message
+                    logging.getLogger(__name__).exception(e.message)
                     return
                 try:
                     newsize = FileUtilities.getsize(self.path)
-                except OSError, e:
+                except OSError as e:
                     from errno import ENOENT
                     if e.errno == ENOENT:
                         # file does not exist
@@ -177,7 +190,7 @@ class Ini:
         self.parameter = parameter
 
     def __str__(self):
-        return 'Command to clean .ini path=%s, section=-%s, parameter=%s ' % \
+        return 'Command to clean .ini path=%s, section=%s, parameter=%s ' % \
             (self.path, self.section, self.parameter)
 
     def execute(self, really_delete):
@@ -211,7 +224,7 @@ class Json:
         self.path = path
         self.address = address
 
-    def __str(self):
+    def __str__(self):
         return 'Command to clean JSON file, path=%s, address=%s ' % \
             (self.path, self.address)
 
@@ -286,8 +299,7 @@ class Winreg:
         self.valuename = valuename
 
     def __str__(self):
-        return 'Command to clean registry, key=%, value=%s ' % \
-            (self.keyname, self.valuename)
+        return 'Command to clean registry, key=%s, value=%s ' % (self.keyname, self.valuename)
 
     def execute(self, really_delete):
         """Execute the Windows registry cleaner"""
@@ -297,10 +309,10 @@ class Winreg:
         ret = None  # return value meaning 'deleted' or 'delete-able'
         if self.valuename:
             _str = '%s<%s>' % (self.keyname, self.valuename)
-            ret = Windows.delete_registry_value(self.keyname,
+            ret = bleachbit.Windows.delete_registry_value(self.keyname,
                                                 self.valuename, really_delete)
         else:
-            ret = Windows.delete_registry_key(self.keyname, really_delete)
+            ret = bleachbit.Windows.delete_registry_key(self.keyname, really_delete)
             _str = self.keyname
         if not ret:
             # Nothing to delete or nothing was deleted.  This return
