@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -54,7 +54,6 @@ if 'posix' == os.name:
     pywinerror = WindowsError
 
 
-
 def open_files_linux():
     return glob.iglob("/proc/*/fd/*")
 
@@ -62,7 +61,7 @@ def open_files_linux():
 def open_files_lsof(run_lsof=None):
     if run_lsof is None:
         def run_lsof():
-            subprocess.check_output(["lsof", "-Fn", "-n"])
+            return subprocess.check_output(["lsof", "-Fn", "-n"])
     for f in run_lsof().split("\n"):
         if f.startswith("n/"):
             yield f[1:]  # Drop lsof's "n"
@@ -71,7 +70,7 @@ def open_files_lsof(run_lsof=None):
 def open_files():
     if sys.platform.startswith('linux'):
         files = open_files_linux()
-    elif 'darwin' == sys.platform:
+    elif 'darwin' == sys.platform or sys.platform.startswith('freebsd'):
         files = open_files_lsof()
     else:
         raise RuntimeError('unsupported platform for open_files()')
@@ -268,7 +267,7 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
             # [Errno 39] Directory not empty
             # https://bugs.launchpad.net/bleachbit/+bug/1012930
             if errno.ENOTEMPTY == e.errno:
-                logger.info("directory is not empty: %s", path)
+                logger.info(_("Directory is not empty: %s"), path)
             else:
                 raise
         except WindowsError as e:
@@ -277,7 +276,7 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
             # Error 145 may happen if the files are scheduled for deletion
             # during reboot.
             if 145 == e.winerror:
-                logger.info("directory is not empty: %s", path)
+                logger.info(_("Directory is not empty: %s"), path)
             else:
                 raise
     elif os.path.isfile(path):
@@ -301,7 +300,7 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
             # unlink
             os.remove(path)
     else:
-        logger.info("special file type cannot be deleted: %s", path)
+        logger.info(_("Special file type cannot be deleted: %s"), path)
 
 
 def ego_owner(filename):
@@ -475,7 +474,7 @@ def guess_overwrite_paths():
     elif 'nt' == os.name:
         localtmp = bleachbit.expandvars('$TMP')
         if not os.path.exists(localtmp):
-            logger.warning('%TMP% does not exist: %s', localtmp)
+            logger.warning(_("%TMP% does not exist: %s"), localtmp)
             localtmp = None
         from bleachbit.Windows import get_fixed_drives
         for drive in get_fixed_drives():
@@ -568,7 +567,30 @@ def truncate_f(f):
     os.fsync(f.fileno())
 
 
-def whitelisted_posix(path, check_realpath = True):
+def uris_to_paths(file_uris):
+    """Return a list of paths from text/uri-list"""
+    import urlparse
+    import urllib
+    assert isinstance(file_uris, (tuple, list))
+    file_paths = []
+    for file_uri in file_uris:
+        if not file_uri:
+            # ignore blank
+            continue
+        parsed_uri = urlparse.urlparse(file_uri)
+        if parsed_uri.scheme == 'file':
+            file_path = urllib.url2pathname(parsed_uri.path)
+            if file_path[2] == ':':
+                # remove front slash for Windows-style path
+                file_path = file_path[1:]
+            file_path_unicode = file_path.decode("utf-8")
+            file_paths.append(file_path_unicode)
+        else:
+            logger.warning('Unsupported scheme: %s', file_uri)
+    return file_paths
+
+
+def whitelisted_posix(path, check_realpath=True):
     """Check whether this POSIX path is whitelisted"""
     from bleachbit.Options import options
     if check_realpath and os.path.islink(path):
@@ -669,7 +691,7 @@ def wipe_contents(path, truncate=True):
         except UnsupportedFileSystemError as e:
             warnings.warn(
                 _('There was at least one file on a file system that does not support advanced overwriting.'), UserWarning)
-            f = open(path, 'wb')
+            f = wipe_write()
         else:
             # The wipe succeed, so prepare to truncate.
             f = open(path, 'wb')
@@ -804,11 +826,13 @@ def wipe_path(pathname, idle=False):
         # Write to OS buffer
         try:
             f.flush()
-        except:
+        except IOError as e:
             # IOError: [Errno 28] No space left on device
             # seen on Microsoft Windows XP SP3 with ~30GB free space but
             # not on another XP SP3 with 64MB free space
-            logger.info("info: exception on f.flush()", exc_info=True)
+            if not e.errno == errno.ENOSPC:
+                logger.error("info: exception on f.flush()")
+
         os.fsync(f.fileno())  # write to disk
         # Remember to delete
         files.append(f)
