@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -252,6 +252,7 @@ class GUI:
         <menu action="File">
             <menuitem action="ShredFiles"/>
             <menuitem action="ShredFolders"/>
+            <menuitem action="ShredClipboard"/>
             <menuitem action="WipeFreeSpace"/>
             <menuitem action="ShredQuit"/>
             <menuitem action="Quit"/>
@@ -412,6 +413,10 @@ class GUI:
             if pynotify.init(APP_NAME):
                 notify = pynotify.Notification('BleachBit', _("Done."),
                                                icon='bleachbit')
+                if 'posix' == os.name and bleachbit.expanduser('~') == '/root':
+                    notify.set_hint("desktop-entry", "bleachbit-root")
+                else:
+                    notify.set_hint("desktop-entry", "bleachbit")
                 notify.show()
                 notify.set_timeout(10000)
 
@@ -424,7 +429,7 @@ class GUI:
                                           link: GuiBasic.open_url(link, self.window, False))
         dialog = gtk.AboutDialog()
         dialog.set_comments(_("Program to clean unnecessary files"))
-        dialog.set_copyright("Copyright (C) 2008-2018 Andrew Ziem")
+        dialog.set_copyright("Copyright (C) 2008-2019 Andrew Ziem")
         try:
             with open(bleachbit.license_filename) as f:
                 dialog.set_license(f.read())
@@ -493,6 +498,29 @@ class GUI:
         pref = PreferencesDialog(self.window, self.cb_refresh_operations)
         pref.run()
 
+    def cb_menu_shred_clipboard(self, action):
+        """Callback for menu option: shred paths from clipboard"""
+        clipboard = gtk.clipboard_get()
+        clipboard.request_targets(self.cb_clipboard_uri_received)
+
+    def cb_clipboard_uri_received(self, clipboard, targets, data):
+        """Callback for when URIs are received from clipboard"""
+        shred_paths = None
+        if 'text/uri-list' in targets:
+            # Linux
+            shred_uris = clipboard.wait_for_contents(
+                'text/uri-list').get_uris()
+            shred_paths = FileUtilities.uris_to_paths(shred_uris)
+        elif 'FileNameW' in targets:
+            # Windows
+            # Use non-GTK+ functions because because GTK+ 2 does not work.
+            shred_paths = Windows.get_clipboard_paths()
+        if shred_paths:
+            self.shred_paths(shred_paths)
+        else:
+            logger.warning(_('No paths found in clipboard.'))
+
+
     def cb_refresh_operations(self):
         """Callback to refresh the list of cleaners"""
         # reload cleaners from disk
@@ -645,20 +673,8 @@ class GUI:
 
     def cb_drag_data_received(self, widget, context, x, y, selection, target_type, time):
         if target_type == self.TARGET_TYPE_TEXT:
-            import urlparse
-            import urllib
-            assert(type(selection.data) is str)
-            file_urls = selection.data.split("\n")
-            file_paths = []
-            for file_url in file_urls:
-                # strip needed to remove "\r"
-                file_url = file_url.strip()
-                parsed_url = urlparse.urlparse(file_url)
-                if parsed_url.scheme == "file":
-                    file_path = urllib.url2pathname(parsed_url.path)
-                    file_path_unicode = file_path.decode("utf-8")
-                    file_paths.append(file_path_unicode)
-
+            file_uris = selection.get_uris()
+            file_paths = FileUtilities.uris_to_paths(file_uris)
             self.shred_paths(file_paths)
 
     def setup_drag_n_drop_widget(self, widget):
@@ -743,6 +759,8 @@ class GUI:
             ('File', None, _('_File')),
             ('Preferences', gtk.STOCK_PREFERENCES, _(
              "Preferences"), None, None, self.cb_preferences_dialog),
+            ('ShredClipboard', None, _("Shred Paths from Clipboard"),
+             None, None, self.cb_menu_shred_clipboard),
             ('Edit', None, _("_Edit")),
             ('HelpContents', gtk.STOCK_HELP, _('Help Contents'), 'F1', None,
              lambda link: GuiBasic.open_url(
