@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,21 +22,25 @@
 Store and retrieve user preferences
 """
 
+from __future__ import absolute_import, print_function
 
+import bleachbit
+from bleachbit import General
+from bleachbit import _
+
+import logging
 import os
 import re
 import traceback
-import ConfigParser
 
-import Common
-import General
+logger = logging.getLogger(__name__)
 
 if 'nt' == os.name:
     from win32file import GetLongPathName
 
 
 boolean_keys = ['auto_hide', 'auto_start', 'check_beta',
-                'check_online_updates', 'first_start', 'shred', 'exit_done', 'delete_confirmation']
+                'check_online_updates', 'first_start', 'shred', 'exit_done', 'delete_confirmation', 'units_iec']
 if 'nt' == os.name:
     boolean_keys.append('update_winapp2')
 
@@ -60,29 +64,32 @@ class Options:
 
     def __init__(self):
         self.purged = False
-        self.config = ConfigParser.SafeConfigParser()
+        self.config = bleachbit.RawConfigParser()
         self.config.optionxform = str  # make keys case sensitive for hashpath purging
+        self.config._boolean_states['t'] = True
+        self.config._boolean_states['f'] = False
         self.restore()
 
     def __flush(self):
         """Write information to disk"""
         if not self.purged:
             self.__purge()
-        if not os.path.exists(Common.options_dir):
-            General.makedirs(Common.options_dir)
-        mkfile = not os.path.exists(Common.options_file)
-        _file = open(Common.options_file, 'wb')
+        if not os.path.exists(bleachbit.options_dir):
+            General.makedirs(bleachbit.options_dir)
+        mkfile = not os.path.exists(bleachbit.options_file)
+        _file = open(bleachbit.options_file, 'wb')
         try:
             self.config.write(_file)
-        except IOError, e:
-            print e
+        except IOError as e:
+            print(e)
             from errno import ENOSPC
             if e.errno == ENOSPC:
-                print "Error: disk is full writing configuration '%s'" % Common.options_file
+                logger.error(
+                    _("Disk was full when writing configuration to file %s"), bleachbit.options_file)
             else:
                 raise
         if mkfile and General.sudo_mode():
-            General.chownself(Common.options_file)
+            General.chownself(bleachbit.options_file)
 
     def __purge(self):
         """Clear out obsolete data"""
@@ -95,7 +102,16 @@ class Options:
                 # restore colon lost because ConfigParser treats colon special
                 # in keys
                 pathname = pathname[0] + ':' + pathname[1:]
-            if not os.path.lexists(pathname):
+                pathname = pathname.decode('utf-8')
+            exists = False
+            try:
+                exists = os.path.lexists(pathname)
+            except:
+                # this deals with corrupt keys
+                # https://www.bleachbit.org/forum/bleachbit-wont-launch-error-startup
+                logger.error(
+                    _("Error checking whether path exists: %s"), pathname)
+            if not exists:
                 # the file does not on exist, so forget it
                 self.config.remove_option('hashpath', option)
 
@@ -112,7 +128,7 @@ class Options:
             option = option[0] + option[2:]
         if option in boolean_keys:
             return self.config.getboolean(section, option)
-        return self.config.get(section, option)
+        return self.config.get(section, option.encode('utf-8'))
 
     def get_hashpath(self, pathname):
         """Recall the hash for a file"""
@@ -168,7 +184,7 @@ class Options:
     def get_tree(self, parent, child):
         """Retrieve an option for the tree view.  The child may be None."""
         option = parent
-        if None != child:
+        if child is not None:
             option += "." + child
         if not self.config.has_option('tree', option):
             return False
@@ -182,7 +198,7 @@ class Options:
     def restore(self):
         """Restore saved options from disk"""
         try:
-            self.config.read(Common.options_file)
+            self.config.read(bleachbit.options_file)
         except:
             traceback.print_exc()
         if not self.config.has_section("bleachbit"):
@@ -190,12 +206,12 @@ class Options:
         if not self.config.has_section("hashpath"):
             self.config.add_section("hashpath")
         if not self.config.has_section("list/shred_drives"):
-            from FileUtilities import guess_overwrite_paths
+            from bleachbit.FileUtilities import guess_overwrite_paths
             try:
                 self.set_list('shred_drives', guess_overwrite_paths())
             except:
-                traceback.print_exc()
-                print 'ERROR: error setting default shred drives'
+                logger.exception(
+                    _("Error when setting the default drives to shred."))
 
         # set defaults
         self.__set_default("auto_hide", True)
@@ -205,30 +221,31 @@ class Options:
         self.__set_default("shred", False)
         self.__set_default("exit_done", False)
         self.__set_default("delete_confirmation", True)
+        self.__set_default("units_iec", False)
 
         if 'nt' == os.name:
             self.__set_default("update_winapp2", False)
 
         if not self.config.has_section('preserve_languages'):
-            lang = Common.user_locale
+            lang = bleachbit.user_locale
             pos = lang.find('_')
             if -1 != pos:
                 lang = lang[0: pos]
             for _lang in set([lang, 'en']):
-                print "info: automatically preserving language '%s'" % lang
+                logger.info(_("Automatically preserving language %s."), lang)
                 self.set_language(_lang, True)
 
         # BleachBit upgrade or first start ever
         if not self.config.has_option('bleachbit', 'version') or \
-                self.get('version') != Common.APP_VERSION:
+                self.get('version') != bleachbit.APP_VERSION:
             self.set('first_start', True)
 
         # set version
-        self.set("version", Common.APP_VERSION)
+        self.set("version", bleachbit.APP_VERSION)
 
     def set(self, key, value, section='bleachbit', commit=True):
         """Set a general option"""
-        self.config.set(section, key, str(value))
+        self.config.set(section, key.encode('utf-8'), str(value))
         if commit:
             self.__flush()
 
@@ -289,7 +306,7 @@ class Options:
         if not self.config.has_section("tree"):
             self.config.add_section("tree")
         option = parent
-        if None != child:
+        if child is not None:
             option = option + "." + child
         if self.config.has_option('tree', option) and not value:
             self.config.remove_option('tree', option)
