@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2019 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,10 +22,12 @@
 Cross-platform, special cleaning operations
 """
 
-import os.path
+from __future__ import absolute_import, print_function
 
-from Options import options
-import FileUtilities
+from bleachbit.Options import options
+from bleachbit import FileUtilities
+
+import os.path
 
 
 def __get_chrome_history(path, fn='History'):
@@ -117,7 +119,7 @@ def delete_chrome_favicons(path):
     ver = __get_chrome_history(path)
     cmds = ""
 
-    if ver in [4, 20, 22, 23, 25, 26, 28, 29, 32]:
+    if ver >= 4:
         # Version 4 includes Chromium 12
         # Version 20 includes Chromium 14, Google Chrome 15, Google Chrome 19
         # Version 22 includes Google Chrome 20
@@ -126,6 +128,8 @@ def delete_chrome_favicons(path):
         # Version 28 is Google Chrome 30
         # Version 29 is Google Chrome 37
         # Version 32 is Google Chrome 51
+        # Version 36 is Google Chrome 60
+        # Version 38 is Google Chrome 64
 
         # icon_mapping
         cols = ('page_url',)
@@ -137,7 +141,7 @@ def delete_chrome_favicons(path):
 
         # favicon images
         cols = ('image_data', )
-        where = "where id not in (select distinct id from icon_mapping)"
+        where = "where icon_id not in (select distinct icon_id from icon_mapping)"
         cmds += __shred_sqlite_char_columns('favicon_bitmaps', cols, where)
 
         # favicons
@@ -203,7 +207,7 @@ def delete_chrome_keywords(path):
     cmds = __shred_sqlite_char_columns('keywords', cols, where)
     cmds += "update keywords set usage_count = 0;"
     ver = __get_chrome_history(path, 'Web Data')
-    if ver >= 43 and ver < 49:
+    if 43 <= ver < 49:
         # keywords_backup table first seen in Google Chrome 17 / Chromium 17 which is Web Data version 43
         # In Google Chrome 25, the table is gone.
         cmds += __shred_sqlite_char_columns('keywords_backup', cols, where)
@@ -244,7 +248,7 @@ def delete_mozilla_url_history(path):
     cmds += __shred_sqlite_char_columns('moz_places', cols, places_suffix)
 
     # delete any orphaned annotations in moz_annos
-    annos_suffix =  "where id in (select moz_annos.id " \
+    annos_suffix = "where id in (select moz_annos.id " \
         "from moz_annos " \
         "left join moz_places " \
         "on moz_annos.place_id = moz_places.id " \
@@ -257,8 +261,9 @@ def delete_mozilla_url_history(path):
     fav_suffix = "where id not in (select favicon_id " \
         "from moz_places where favicon_id is not null ); "
 
-    cols = ('url', 'data')
-    cmds += __shred_sqlite_char_columns('moz_favicons', cols, fav_suffix)
+    if __sqlite_table_exists(path, 'moz_favicons'):
+        cols = ('url', 'data')
+        cmds += __shred_sqlite_char_columns('moz_favicons', cols, fav_suffix)
 
     # delete any orphaned history visits
     cmds += "delete from moz_historyvisits where place_id not " \
@@ -313,10 +318,7 @@ def get_chrome_bookmark_ids(history_path):
 
 def get_chrome_bookmark_urls(path):
     """Return a list of bookmarked URLs in Google Chrome/Chromium"""
-    try:
-        import json
-    except:
-        import simplejson as json
+    import json
 
     # read file to parser
     js = json.load(open(path, 'r'))
@@ -328,13 +330,13 @@ def get_chrome_bookmark_urls(path):
     def get_chrome_bookmark_urls_helper(node):
         if not isinstance(node, dict):
             return
-        if not node.has_key('type'):
+        if 'type' not in node:
             return
         if node['type'] == "folder":
             # folders have children
             for child in node['children']:
                 get_chrome_bookmark_urls_helper(child)
-        if node['type'] == "url" and node.has_key('url'):
+        if node['type'] == "url" and 'url' in node:
             urls.append(node['url'])
 
     # find bookmarks
