@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -89,7 +89,9 @@ def browse_file(_, title):
 def browse_files(_, title):
     """Ask the user to select files.  Return full paths"""
     try:
+        # The File parameter is a hack to increase the buffer length.
         ret = win32gui.GetOpenFileNameW(None,
+                                        File = '\x00' * 10240,
                                         Flags=win32con.OFN_ALLOWMULTISELECT
                                         | win32con.OFN_EXPLORER
                                         | win32con.OFN_FILEMUSTEXIST
@@ -352,6 +354,19 @@ def get_autostart_path():
     return os.path.join(startupdir, 'bleachbit.lnk')
 
 
+def get_clipboard_paths():
+    """Return a tuple of Unicode pathnames from the clipboard"""
+    import win32clipboard
+    win32clipboard.OpenClipboard()
+    path_list = ()
+    try:
+        path_list = win32clipboard.GetClipboardData(win32clipboard.CF_HDROP)
+    except TypeError:
+        pass
+    finally:
+        win32clipboard.CloseClipboard()
+    return path_list
+
 def get_fixed_drives():
     """Yield each fixed drive"""
     for drive in win32api.GetLogicalDriveStrings().split('\x00'):
@@ -434,11 +449,12 @@ def get_recycle_bin():
     for item in h:
         path = h.GetDisplayNameOf(item, shellcon.SHGDN_FORPARSING)
         if os.path.isdir(path):
-            for child in FileUtilities.children_in_directory(path, True):
-                yield child
-            yield path
-        else:
-            yield path
+            if not is_link(path):
+                # Return the contents of a normal directory, but do
+                # not recurse Windows symlinks in the Recycle Bin.
+                for child in FileUtilities.children_in_directory(path, True):
+                    yield child
+        yield path
 
 
 def get_windows_version():
@@ -448,6 +464,17 @@ def get_windows_version():
     return Decimal(vstr)
 
 
+def is_link(path):
+    """Check whether the path is a link
+
+    On Python 2.7 the function os.path.islink() always returns False,
+    so this is needed
+    """
+    FILE_ATTRIBUTE_REPARSE_POINT = 0x400
+    attr = windll.kernel32.GetFileAttributesW(unicode(path))
+    return bool(attr & FILE_ATTRIBUTE_REPARSE_POINT)
+
+
 def is_process_running(name):
     """Return boolean whether process (like firefox.exe) is running"""
 
@@ -593,6 +620,10 @@ def setup_environment():
         logger.exception('exception identifying LocalAppDataLow')
     else:
         set_environ('LocalAppDataLow', path)
+    # %cd% can be helpful for cleaning portable applications when
+    # BleachBit is portable. It is the same variable name as defined by
+    # cmd.exe .
+    set_environ('cd', os.getcwd())
 
 
 def split_registry_key(full_key):
