@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2019 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -40,7 +40,7 @@ else:
     from ConfigParser import RawConfigParser, NoOptionError, SafeConfigParser
 
 
-APP_VERSION = "2.0"
+APP_VERSION = "2.2"
 APP_NAME = "BleachBit"
 APP_URL = "https://www.bleachbit.org"
 
@@ -158,6 +158,7 @@ def expanduser(path):
             home = os.path.join(h_drive, h_path)
         final = final.replace('~user/', '')
         final = final.replace('~/', '')
+        final = final.replace('~\\', '')
         final = final.replace('~', '')
         final = os.path.join(home, final)
     return final
@@ -195,6 +196,8 @@ elif sys.platform == 'win32':
     system_cleaners_dir = os.path.join(bleachbit_exe_path, 'share\\cleaners\\')
 elif sys.platform[:6] == 'netbsd':
     system_cleaners_dir = '/usr/pkg/share/bleachbit/cleaners'
+elif sys.platform.startswith('openbsd') or sys.platform.startswith('freebsd'):
+    system_cleaners_dir = '/usr/local/share/bleachbit/cleaners'
 else:
     system_cleaners_dir = None
     logger.warning('unknown system cleaners directory for platform %s ', sys.platform)
@@ -206,10 +209,13 @@ if portable_mode:
         os.path.join(bleachbit_exe_path, '../cleaners'))
 
 # application icon
-__icons = ('/usr/share/pixmaps/bleachbit.png',  # Linux
-           os.path.join(bleachbit_exe_path, 'share\\bleachbit.png'),  # Windows
-           '/usr/pkg/share/pixmaps/bleachbit.png',  # NetBSD
-           os.path.normpath(os.path.join(bleachbit_exe_path, '../bleachbit.png')))  # local
+__icons = (
+    '/usr/share/pixmaps/bleachbit.png',  # Linux
+    os.path.join(bleachbit_exe_path, 'share\\bleachbit.png'),  # Windows
+    '/usr/pkg/share/pixmaps/bleachbit.png',  # NetBSD
+    '/usr/local/share/pixmaps/bleachbit.png',  # FreeBSD and OpenBSD
+    os.path.normpath(os.path.join(bleachbit_exe_path, '../bleachbit.png')),  # local
+)
 appicon_path = None
 for __icon in __icons:
     if os.path.exists(__icon):
@@ -228,6 +234,9 @@ else:
         locale_dir = os.path.join(bleachbit_exe_path, 'share\\locale\\')
     elif sys.platform[:6] == 'netbsd':
         locale_dir = "/usr/pkg/share/locale/"
+    elif (sys.platform.startswith('openbsd') or
+          sys.platform.startswith('freebsd')):
+        locale_dir = "/usr/local/share/locale/"
 
 
 # launcher
